
clear; tic; format short;

%  WABA: Within And Between Groups Analysis

%  Sources:
%  Dansereau, Alutto, & Yammarino (1984). Theory Testing in Organizational
%  Behavior. Prentice-Hall.
%  Yammarino & Markham (1992). Journal of Applied Psychology, 77, 168-176.
%  www.LevelsOfAnalysis.com

%  Prepare a raw data matrix for analysis,
%  where rows = cases, & columns = variables;
%  Each row contains the data from a single individual;
%  The values/variables in each row are separated from each other by one or
%  more spaces; Cases with missing values are not permitted in the data file.

%  The first value in each row (i.e., the first column of values in the data file)
%  must be the individuals' group numbers/codes, which must be integers.  
%  The program sorts individuals into groups on the basis of these numbers/codes.
%  Variable scores appear in subsequent columns.

%  Multiple Variable Analyses are conducted when the number of variables = 3,
%  which is a limit that is determined by Hotelling's t-test for
%  dependent correlations.  For MVAs involving more than three variables,
%  simply run this program again using new/different combinations of three variables.

%  Use the following name for the raw data matrix: "data"


% DETECT Data Set A, from www.LevelsOfAnalysis.com
% variables: DYAD NEGOT SATIS PERF TASKCLR
data = [
  1    14    14   32     10
  1    22    28   33     11
  3    13    16   39     12
  3    26    26   39     12
  4    26    22   36     13
  4    16    20   35     13
  6    16    16   26     14
  6    24    22   27     16
  7    23    24   27     16
  7    15    22   26     16
  2    20    23   22     17
  2    13    19   22     19
  5    18    15   20     20
  5    22    23   20     22
  8    21    25    4     30
  8    14    17    3     33
  9    27    24    2     32
  9    14    19    4     32
 10    25    25   18     19
 10    15    17   20     18
 31    14    14   32     10
 31    22    28   33     11
 33    13    16   39     12
 33    26    26   39     12
 34    26    22   36     13
 34    16    20   35     13
 36    16    16   26     14
 36    24    22   27     16
 37    23    24   27     16
 37    15    22   26     16
 32    20    23   22     17
 32    13    19   22     19
 35    18    15   20     20
 35    22    23   20     22
 38    21    25    4     30
 38    14    17    3     33
 39    27    24    2     32
 39    14    19    4     32
 40    25    25   18     19
 40    15    17   20     18  ];
% remove one variable from the above data matrix in order to
% conduct Multiple Variable Analyses (the # variables must = 3)
% data = data(:,1:4);


% Done.  The program will can now analyze the data.

totmn = mean( data(:,2:size(data,2)) ,1   );

totdev  = repmat(-9999, size(data,1), size(data,2)-1);
withdev = repmat(-9999, size(data,1), size(data,2)-1);
betwdev = repmat(-9999, size(data,1), size(data,2)-1);

grpns    = repmat(-9999, 1, 1);

first = 1; ngrps = 0;

for luper1 = 2:size(data,1)
if data((luper1),1) ~= data((luper1-1),1)  |  luper1 == size(data,1);  
if luper1 ~= size(data,1); last = luper1 - 1;end
if luper1 == size(data,1); last = luper1    ;end
ngrps = ngrps + 1;
cmean = mean( data(first:last , 2:size(data,2)) , 1 );

grpns = [ grpns ; (last - first) + 1 ];

for luper2 = first : last
totdev(luper2,:)  = data(luper2,2:size(data,2)) - totmn;
withdev(luper2,:) = data(luper2,2:size(data,2)) - cmean;
betwdev(luper2,:) = cmean - totmn;
end
first = luper1;
end;end

cormat = corrcoef( [ totdev betwdev withdev ] );
nvars = size(cormat,1) / 3;
nss = size(data,1);

disp([' ']);disp([' ']);disp(['WABA']);disp([' ']);disp([' ']);
disp(['Number of Individual Cases in the Data File = ' num2str(nss) ]);disp([' ']);
disp(['Number of Groups in the Data File = ' num2str(ngrps) ]);disp([' ']);
disp(['Number of Variables for the Analyses = ' num2str(nvars) ]);disp([' ']);
disp(['All Significance Tests Are Two-Tailed' ]);disp([' ']);disp([' ']);
disp(['Variable Means and Standard Deviations:']);disp([' ']);
disp(['   Variable    Mean    Std.Dev.']);
disp([ (1:nvars)' mean(data(:,2:size(data,2)),1)' ...
       std((data(:,2:size(data,2))))' ]);disp([' ']);

% Within- and Between-Groups Analysis of Variance
etabetw = diag( cormat( nvars+1:nvars*2 , 1:nvars) );
etawith = diag( cormat( nvars*2+1:nvars*3 , 1:nvars) );
etasq = [ etabetw etawith ].^2;
% E tests
etests = etabetw ./ etawith;
% F tests
ftrad = (etests.^2) * ( (nss - ngrps) / (ngrps - 1) );
fcrctd = 1 ./ ftrad;
% dfs for the F values (Dansereau et al 1984, p. 125, 128, 172)
dfnumt = repmat((ngrps - 1),nvars,1);
dfdemt = repmat((nss - ngrps),nvars,1);
dfnumc = repmat((nss - ngrps),nvars,1);
dfdemc = repmat((ngrps - 1),nvars,1);
% sig levels for F
pftrad  = 1 - fp(abs(ftrad),dfnumt,dfdemt);
pfcrctd = 1 - fp(abs(fcrctd),dfnumc,dfdemc);
% for displaying only the appropriate F results
fetas = [ (1:nvars)' ftrad dfnumt dfdemt pftrad ];
for lupevars = 1:nvars
if etawith(lupevars,1) > etabetw(lupevars,1);  
fetas(lupevars,2) = fcrctd(lupevars,1);
fetas(lupevars,3) = dfnumc(lupevars,1);
fetas(lupevars,4) = dfdemc(lupevars,1);
fetas(lupevars,5) = pfcrctd(lupevars,1);end;end

disp([' ']);disp(['Within-Groups and Between-Groups Etas and Eta-squared values:']);
disp([' ']);disp(['   Variable  Eta-Betw  Eta-With  Eta-Bsq.   Eta-Wsq.'])
disp([ (1:nvars)' etabetw etawith etasq]);disp([' ']);disp([' ']);
disp(['The above Eta-Betw and Eta-With values for each variable are tested relative to one']);
disp(['another with E tests of practical significance and F tests of statistical significance:']);
disp([' ']);disp(['   Variable   E Test      F      df num.   df dem.      p'])
disp([ fetas(:,1) etests fetas(:, 2:5) ]);disp([' ']);
disp(['    E-test practical significance criteria & inductions:']);
disp(['    E >= 1.30323 = Wholes - 15;     E >= 1.73205 = Wholes - 30;' ])
disp(['    E <= 0.76733 = Parts  - 15;     E <= 0.57735 = Parts  - 30;' ])
disp([' ']);disp([' ']);

% Other indices of within-group agreement: Intraclass Correlations
grpns = grpns( 2:(size(grpns,1)), :);
ssb =  sum( (betwdev.^2) , 1);
dfb = ngrps - 1;
msb = ssb / dfb;
ssw =  sum( (withdev.^2) , 1);
dfw = sum(grpns - 1);
msw = ssw / dfw;
icc1 = (msb - msw) ./ (msb + ( (mean(grpns)-1) * msw));
icc2 = (msb - msw) ./ msb;
disp(['Other Indices of Within-Group Agreement: Intraclass Correlations']);
disp([' ']);disp(['   Variable   ICC(1)    ICC(2)'])
disp([ (1:nvars)' icc1' icc2']);disp([' ']);
disp(['   Sources:']);
disp(['   Bliese (2000), in Klein et al, Multilevel theory, research, and methods in organizations.'])
disp(['   Shrout & Fleiss, 1979, Psychological Bulletin, 86, 420-428.']);
disp(['   McGraw & Wong, 1996, Psychological Methods, 1, 30-46.']);disp([' ']);

% Within- and Between-Groups Analysis of Covariance Analyses
rbetw = cormat( nvars+1:nvars*2   , nvars+1:nvars*2);
rwith = cormat( nvars*2+1:nvars*3 , nvars*2+1:nvars*3);
rbw = repmat(-9999,1, 4);
for luper = 1:nvars-1; for lupec = luper+1:nvars;
rbw = [rbw;  luper lupec rbetw(luper,lupec) rwith(luper,lupec) ];end;end
rbw = rbw(2:size(rbw,1),:);
% A test
atests = asin(sqrt(1-(rbw(:,4).^2))) - asin(sqrt(1-(rbw(:,3).^2)));
% Z test
zbw = repmat(-9999,size(rbw,1), 1);
if ngrps > 3 
for luper = 1:size(rbw,1);
if abs(rbw(luper,3)) < 1  &  abs(rbw(luper,4)) < 1  
zbxy =abs( 0.5* ((log (1+abs(rbw(luper,3)))) - (log (1-abs(rbw(luper,3))))) );
zwxy =abs( 0.5* ((log (1+abs(rbw(luper,4)))) - (log (1-abs(rbw(luper,4))))) );
zbw(luper,1) = (zbxy - zwxy) / sqrt( (1/(nss-ngrps-2))+(1/(ngrps-3)) );
end;end;end
pzbw = 1 - normp(abs(zbw)) ;
% R test of practical sigificance
rbig = repmat(-9999,size(rbw,1), 2);
for luper = 1:size(rbw,1); for lupec = 3:4;
if abs(rbw(luper,lupec)) < 1
% rbig formula from Dansereau 1984 p 131
rbig(luper,lupec-2) = abs( rbw(luper,lupec) ./ sqrt ( 1 - rbw(luper,lupec).^2 ) );
% rbig formula from Yamarino & Markham, 1992 p 170  = slightly diff
% rbig(luper,lupec-2) = rbw(luper,lupec) ./ ( ( (1 - (rbw(luper,lupec).^2) ).^.6667) );
end;end;end
% t-tests of statistical significance (Yamarino & Markham, 1992 p 170)
tb = rbig(:,1) * sqrt(ngrps-2);
dftb = ngrps - 2;
tw = rbig(:,2) * sqrt(nss-ngrps-1);
dftw = nss - ngrps - 1;
ptb = repmat(-9999, size(tb,1), 1 );
ptw = repmat(-9999, size(tw,1), 1 );
for luper = 1:size(tb,1)
ptb(luper,1) = (1 - tp(abs(tb(luper,1)),dftb)) * 2;
ptw(luper,1) = (1 - tp(abs(tw(luper,1)),dftw)) * 2;
end

disp([' ']);disp(['Within-Groups and Between-Groups Correlations:']);
disp([' ']);disp(['     Var#   &  Var#     r-Betw    r-With    A-test    Z-test      p']);
disp([rbw atests zbw pzbw]);disp([' ']);
disp(['    A-tests represent practical, and Z-tests represent statistical, assessments of']);
disp(['    the differences between the within-groups and between-groups correlations.']);disp([' ']);
disp(['    A-test practical significance criteria & inductions:']);
disp(['    A >=  0.2618 = Wholes - 15;     A >=  0.5236 = Wholes - 30;' ])
disp(['    A <= -0.2618 = Parts  - 15;     A <= -0.5236 = Parts  - 30;' ])
disp([' ']);disp(['    -9999 indicates the Z test could not be computed.']);disp([' ']);
disp([' ']);disp(['Between-Groups Correlation Tests of Practical (R) and Statistical (t) Significance'])
disp([' ']);disp(['     Var#   &  Var#       R       t-test      p'])
disp([ rbw(:,1:2) rbig(:,1) tb ptb ]);disp([' ']);
disp(['    R-test practical significance criteria & inductions:']);
disp(['    R >=  0.26795 = S - 15;     R >=  0.57735 = S - 30;' ])
disp([' ']);disp(['    Degrees of Freedom for t-tests: 1,' num2str(dftb) ]);
disp(['    A value of -9999 indicates the t-test could not be computed.']);disp([' ']);
disp([' ']);disp(['Within-Groups Correlation Tests of Practical (R) and Statistical (t) Significance'])
disp([' ']);disp(['     Var#   &  Var#       R       t-test      p'])
disp([ rbw(:,1:2) rbig(:,2) tw ptw ]);disp([' ']);
disp(['    R-test practical significance criteria & inductions:']);
disp(['    R >=  0.26795 = S - 15;     R >=  0.57735 = S - 30;' ])
disp([' ']);disp(['    Degrees of Freedom for t-tests: 1,' num2str(dftw) ]);
disp(['    A value of -9999 indicates the t-test could not be computed.']);disp([' ']);
disp([' ']);disp([' ']);

% Within- and Between-Groups Components and Raw (Total) Correlations
compons = rbw;
for luper = 1:size(rbw,1)
compons(luper,3) = etabetw(rbw(luper,1),1) * etabetw(rbw(luper,2),1) * rbw(luper,3);
compons(luper,4) = etawith(rbw(luper,1),1) * etawith(rbw(luper,2),1) * rbw(luper,4);
end
compons(:,5) = compons(:,3) + compons(:,4);
% R test of practical sigificance of the total correlations
rbig2 = repmat(-9999,size(compons,1), 1);
for luper = 1:size(compons,1);
if abs(compons(luper,5)) < 1
% rbig formula from Dansereau 1984 p 131
rbig2(luper,1) = abs( compons(luper,5) ./ sqrt ( 1 - compons(luper,5).^2 ) );
% rbig formula from Yamarino & Markham, 1992 p 170  = slightly diff
% rbig2(luper,lupec-2) = rbw(luper,lupec) ./ ( ( (1 - (rbw(luper,lupec).^2) ).^.6667) );
end;end
% t-tests of statistical significance of the total correlations Dansereau et al 1984 p 119
ttot = rbig2 * sqrt(nss-2);
dftot = nss - 2;
ptot = repmat(-9999, size(ttot,1), 1 );
for luper = 1:size(ttot,1)
ptot(luper,1) = (1 - tp(abs(ttot(luper,1)),dftot)) * 2;
end
% A test of the between vs within components difference 
aradiansc = asin(sqrt(1-(compons(:,4).^2))) - asin(sqrt(1-(compons(:,3).^2)));
% Z test of the between vs within components difference 
zbwc = repmat(-9999,size(compons,1), 1);
if ngrps > 3 
for luper = 1:size(compons,1);
if abs(compons(luper,3)) < 1  &  abs(compons(luper,4)) < 1  
zbxyc =abs( 0.5* ((log (1+abs(compons(luper,3)))) - (log (1-abs(compons(luper,3))))) );
zwxyc =abs( 0.5* ((log (1+abs(compons(luper,4)))) - (log (1-abs(compons(luper,4))))) );
zbwc(luper,1) = (zbxyc - zwxyc) / sqrt( (1/(nss-ngrps-2))+(1/(ngrps-3)) );
end;end;end
pzbwc = (1 - normp(abs(zbwc))) * 2;


disp([' ']);disp(['Within-Groups and Between-Groups WABA Components and Raw/Total Correlation(s):']);
disp([' ']);disp(['     Var#  &   Var#      Betw      With    Raw/Total']);
disp(compons);disp([' ']);
disp([' ']);disp(['Practical (R) and Statistical (t) Significance of the Raw/Total correlations:'])
disp([' ']);disp(['     Var#  &   Var#       R       t-test      p']);
disp([ compons(:,1:2) rbig2 ttot ptot ]);disp([' ']);
disp(['    R-test practical significance criteria & inductions:']);
disp(['    R >=  0.26795 = S - 15;     R >=  0.57735 = S - 30;' ])
disp([' ']);disp(['    Degrees of Freedom for t-tests: 1,' num2str(dftot) ]);
disp(['    A value of -9999 indicates the t-test could not be computed.']);disp([' ']);disp([' ']);
disp(['The Within-Groups and Between-Groups WABA Components are tested relative to']);
disp(['one another with A tests of practical Z tests of statistical significance.']);disp([' ']);
disp(['     Var#  &   Var#     A-test    Z-test      p']);
disp([compons(:,1:2) aradiansc zbwc pzbwc]);disp([' ']);
disp(['    A-test practical significance criteria & inductions:']);
disp(['    A >=  0.2618 = Wholes - 15;     A >=  0.5236 = Wholes - 30;' ])
disp(['    A <= -0.2618 = Parts  - 15;     A <= -0.5236 = Parts  - 30;' ])
disp([' ']);disp(['    -9999 indicates the Z test could not be computed.']);
disp([' ']);disp([' ']);


% Multiple Variable Analysis
if nvars ~= 3;
disp([' ']);disp([' ']);disp([' ']);
disp(['Multiple Variable Analysis was not conducted because the number of variables was not = 3']);end
if nvars == 3


disp([' ']);disp([' ']);disp([' ']);disp([' ']);
disp(['Multiple Variable Analysis Using Within-Groups Correlations']);
disp([' ']);
disp(['Sources:']); 
disp(['Dansereau, Alutto, & Yammarino (1984). Theory Testing in Organizational Behavior. Prentice-Hall.']); 
disp(['Yammarino (1998). Multivariate aspects of the varient/WABA approach. Leadership Quarterly, 9, 203-227.']); 
disp(['www.LevelsOfAnalysis.com']); 
disp([' ']);
disp(['Correlations are compared using both']);
disp(['"A" tests of practical significance & Hotelling t-tests of statistical significance.']);

dum = rbw;

for luper = 1:3

rxy =  dum(1,4);
rxz =  dum(2,4);
ryz =  dum(3,4);

disp([' ']);disp([' ']);
disp(['The Within correlation between var #' num2str(dum(1,1)) ...
      '  and var #' num2str(dum(1,2))  '   is = ' num2str(rxy) ]); 
disp([' ']);
disp(['The Within correlation between var #' num2str(dum(2,1)) ...
      '  and var #' num2str(dum(2,2)) '   is = ' num2str(rxz) ]); 
disp([' ']);

% "A" test of the difference between the correlations; Dansereau et al, 1984, p 140
atest = acos(abs(rxy)) - acos(abs(rxz));
adegs = atest * 57.29578;
patest = 'n.s.';
if abs(adegs) > 15; patest = '15 degree test';end
if abs(adegs) > 30; patest = '30 degree test';end
disp(['"A" test:   A = ' num2str(abs(atest)) '   p-level:  ' patest ]);
disp([' ']);

% Hotelling's t-test of the difference between the correlations; Dansereau et al, 1984, p 141
hotest = ( abs(rxy) - abs(rxz) ) * sqrt(  ( (ngrps-2)*(1+abs(ryz)) ) / ( 2 * ( 1 - ryz.^2 ...
         - rxy.^2 - rxz.^2 + 2 * abs(ryz) * abs(rxy) * abs(rxz) ) ) );
dfhotest = ngrps-2;
photest = (1 - tp(abs(hotest),dfhotest) ) * 2;
disp(['Hotelling t-test:  t = ' num2str(hotest) '  df = ' num2str(dfhotest) '  p-level =  ' num2str(photest) ]);
disp([' ']);

% rotating the rows of the correlation data matrix
dum = [ dum(3,:) ; dum ]; dum = dum(1:3,:);
end

disp([' ']);disp([' ']);disp([' ']);disp([' ']);
disp(['Multiple Variable Analysis Using Between-Groups Correlations']);
disp([' ']);
disp(['Sources:']); 
disp(['Dansereau, Alutto, & Yammarino (1984). Theory Testing in Organizational Behavior. Prentice-Hall.']); 
disp(['Yammarino (1998). Multivariate aspects of the varient/WABA approach. Leadership Quarterly, 9, 203-227.']); 
disp(['www.LevelsOfAnalysis.com']); 
disp([' ']);
disp(['Correlations are compared using both']);
disp(['"A" tests of practical significance & Hotelling t-tests of statistical significance.']);

dum = rbw;

for luper = 1:3

rxy =  dum(1,3);
rxz =  dum(2,3);
ryz =  dum(3,3);

disp([' ']);disp([' ']);
disp(['The Between correlation between var #' num2str(dum(1,1)) ...
      '  and var #' num2str(dum(1,2))  '   is = ' num2str(rxy) ]); 
disp([' ']);
disp(['The Between correlation between var #' num2str(dum(2,1)) ...
      '  and var #' num2str(dum(2,2)) '   is = ' num2str(rxz) ]); 
disp([' ']);

% "A" test of the difference between the correlations; Dansereau et al, 1984, p 140
atest = acos(abs(rxy)) - acos(abs(rxz));
adegs = atest * 57.29578;
patest = 'n.s.';
if abs(adegs) > 15; patest = '15 degree test';end
if abs(adegs) > 30; patest = '30 degree test';end
disp(['"A" test:   A = ' num2str(abs(atest)) '   p-level:  ' patest ]);
disp([' ']);

% Hotelling's t-test of the difference between the correlations; Dansereau et al, 1984, p 141
hotest = ( abs(rxy) - abs(rxz) ) * sqrt(  ( (ngrps-3)*(1+abs(ryz)) ) / ( 2 * ( 1 - ryz.^2 ...
         - rxy.^2 - rxz.^2 + 2 * abs(ryz) * abs(rxy) * abs(rxz) ) ) );
dfhotest = ngrps - 3;
photest = (1 - tp(abs(hotest),dfhotest) ) * 2;
disp(['Hotelling t-test:  t = ' num2str(hotest) '  df = ' num2str(dfhotest) '  p-level =  ' num2str(photest) ]);
disp([' ']);

% rotating the rows of the correlation data matrix
dum = [ dum(3,:) ; dum ]; dum = dum(1:3,:);
end

end




